@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2019 pyglet contributors
+# Copyright (c) 2008-2020 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -110,57 +110,17 @@ Multiple and derived clocks potentially allow you to separate "game-time" and
 "wall-time", or to synchronise your clock to an audio or video stream instead
 of the system clock.
 """
-from __future__ import print_function
-from __future__ import division
-from builtins import range
-from builtins import object
 
-import sys
 import time
-import ctypes
+
 from operator import attrgetter
 from heapq import heappush, heappop, heappushpop
 from collections import deque
 
-import pyglet.lib
 from pyglet import compat_platform
 
 
-__docformat__ = 'restructuredtext'
-__version__ = '$Id$'
-
-
-if sys.version_info[:2] < (3, 5):
-    # PYTHON2 - remove these legacy classes:
-
-    if compat_platform in ('win32', 'cygwin'):
-
-        class _ClockBase(object):
-            def sleep(self, microseconds):
-                time.sleep(microseconds * 1e-6)
-
-        _default_time_function = time.clock
-
-    else:
-        _c = pyglet.lib.load_library('c')
-        _c.usleep.argtypes = [ctypes.c_ulong]
-
-        class _ClockBase(object):
-            def sleep(self, microseconds):
-                _c.usleep(int(microseconds))
-
-        _default_time_function = time.time
-
-else:
-
-    class _ClockBase(object):
-        def sleep(self, microseconds):
-            time.sleep(microseconds * 1e-6)
-
-    _default_time_function = time.perf_counter
-
-
-class _ScheduledItem(object):
+class _ScheduledItem:
     __slots__ = ['func', 'args', 'kwargs']
 
     def __init__(self, func, args, kwargs):
@@ -169,7 +129,7 @@ class _ScheduledItem(object):
         self.kwargs = kwargs
 
 
-class _ScheduledIntervalItem(object):
+class _ScheduledIntervalItem:
     __slots__ = ['func', 'interval', 'last_ts', 'next_ts', 'args', 'kwargs']
 
     def __init__(self, func, interval, last_ts, next_ts, args, kwargs):
@@ -187,7 +147,7 @@ class _ScheduledIntervalItem(object):
             return self.next_ts < other
 
 
-class Clock(_ClockBase):
+class Clock:
     """Class for calculating and limiting framerate.
 
     It is also used for calling scheduled functions.
@@ -199,6 +159,7 @@ class Clock(_ClockBase):
     #: busy-waiting.  Lower values mean the process sleeps more often, but is
     #: prone to over-sleep and run at a potentially lower or uneven framerate
     #: than desired.
+
     #: On Windows, MIN_SLEEP is larger because the default timer resolution
     #: is set by default to 15 .6 ms.
     MIN_SLEEP = 0.008 if compat_platform in ('win32', 'cygwin') else 0.005
@@ -216,7 +177,7 @@ class Clock(_ClockBase):
     # If True, a sleep(0) is inserted on every tick.
     _force_sleep = False
 
-    def __init__(self, time_function=_default_time_function):
+    def __init__(self, time_function=time.perf_counter):
         """Initialise a Clock, with optional custom time function.
 
         :Parameters:
@@ -230,15 +191,19 @@ class Clock(_ClockBase):
         self.time = time_function
         self.next_ts = self.time()
         self.last_ts = None
-        self.times = deque()
 
+        # Used by self.get_fps to show update frequency
+        self.times = deque()
         self.cumulative_time = 0
+        self.window_size = 60
 
         self._schedule_items = []
         self._schedule_interval_items = []
         self._current_interval_item = None
 
-        self.window_size = 60
+    @staticmethod
+    def sleep(microseconds):
+        time.sleep(microseconds * 1e-6)
 
     def update_time(self):
         """Get the elapsed time since the last call to `update_time`.
@@ -410,7 +375,7 @@ class Clock(_ClockBase):
         .. versionadded:: 1.1
         """
         if self._schedule_items or not sleep_idle:
-                return 0.0
+            return 0.0
 
         if self._schedule_interval_items:
             return max(self._schedule_interval_items[0].next_ts - self.time(), 0.0)
@@ -418,13 +383,14 @@ class Clock(_ClockBase):
         return None
 
     def get_fps(self):
-        """Get the average FPS of recent history.
+        """Get the average clock update frequency of recent history.
 
-        The result is the average of a sliding window of the last "n" frames,
+        The result is the average of a sliding window of the last "n" updates,
         where "n" is some number designed to cover approximately 1 second.
+        This is **not** the Window redraw rate.
 
         :rtype: float
-        :return: The measured frames per second.
+        :return: The measured updates per second.
         """
         if not self.cumulative_time:
             return 0
@@ -688,9 +654,16 @@ def get_sleep_time(sleep_idle):
 
 
 def get_fps():
-    """Return the current measured FPS of the default clock.
+    """Get the average clock update frequency.
+
+    The result is the sliding average of the last "n" updates,
+    where "n" is some number designed to cover approximately 1
+    second. This is the internal clock update rate, **not** the
+    Window redraw rate. Platform events, such as moving the
+    mouse rapidly, will cause the clock to refresh more often.
 
     :rtype: float
+    :return: The measured updates per second.
     """
     return _default.get_fps()
 
