@@ -1,6 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
+# Copyright (c) 2008-2020 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,9 +32,8 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
-# $Id:$
 
-'''Manage related vertex attributes within a single vertex domain.
+"""Manage related vertex attributes within a single vertex domain.
 
 A vertex "domain" consists of a set of attribute descriptions that together
 describe the layout of one or more vertex buffers which are used together to
@@ -54,24 +54,18 @@ to the underlying buffers automatically.
 The entire domain can be efficiently drawn in one step with the
 :py:meth:`VertexDomain.draw` method, assuming all the vertices comprise
 primitives of the same OpenGL primitive mode.
-'''
-from builtins import zip
-from builtins import range
-from builtins import object
+"""
 
-__docformat__ = 'restructuredtext'
-__version__ = '$Id: $'
-
-import ctypes
 import re
+import ctypes
 
 from pyglet.gl import *
 from pyglet.graphics import allocation, vertexattribute, vertexbuffer
 
-_usage_format_re = re.compile(r'''
+_usage_format_re = re.compile(r"""
     (?P<attribute>[^/]*)
     (/ (?P<usage> static|dynamic|stream|none))?
-''', re.VERBOSE)
+""", re.VERBOSE)
 
 _gl_usages = {
     'static': GL_STATIC_DRAW,
@@ -80,6 +74,7 @@ _gl_usages = {
     'none': GL_STREAM_DRAW_ARB, # Force no VBO
 }
 
+
 def _nearest_pow2(v):
     # From http://graphics.stanford.edu/~seander/bithacks.html#RoundUpPowerOf2
     # Credit: Sean Anderson
@@ -91,8 +86,9 @@ def _nearest_pow2(v):
     v |= v >> 16
     return v + 1
 
-def create_attribute_usage(format):
-    '''Create an attribute and usage pair from a format string.  The
+
+def create_attribute_usage(fmt):
+    """Create an attribute and usage pair from a format string.  The
     format string is as documented in `pyglet.graphics.vertexattribute`, with
     the addition of an optional usage component::
 
@@ -111,8 +107,8 @@ def create_attribute_usage(format):
         4-byte color attribute, for static usage
 
     :return: attribute, usage
-    '''
-    match = _usage_format_re.match(format)
+    """
+    match = _usage_format_re.match(fmt)
     attribute_format = match.group('attribute')
     attribute = vertexattribute.create_attribute(attribute_format)
     usage = match.group('usage')
@@ -123,38 +119,39 @@ def create_attribute_usage(format):
         usage = GL_DYNAMIC_DRAW
         vbo = True
 
-    return (attribute, usage, vbo)
+    return attribute, usage, vbo
+
 
 def create_domain(*attribute_usage_formats):
-    '''Create a vertex domain covering the given attribute usage formats.
+    """Create a vertex domain covering the given attribute usage formats.
     See documentation for :py:func:`create_attribute_usage` and
     :py:func:`pyglet.graphics.vertexattribute.create_attribute` for the grammar
     of these format strings.
 
     :rtype: :py:class:`VertexDomain`
-    '''
-    attribute_usages = [create_attribute_usage(f) \
-                        for f in attribute_usage_formats]
+    """
+    attribute_usages = [create_attribute_usage(f) for f in attribute_usage_formats]
     return VertexDomain(attribute_usages)
 
+
 def create_indexed_domain(*attribute_usage_formats):
-    '''Create an indexed vertex domain covering the given attribute usage
+    """Create an indexed vertex domain covering the given attribute usage
     formats.  See documentation for :py:class:`create_attribute_usage` and
     :py:func:`pyglet.graphics.vertexattribute.create_attribute` for the grammar
     of these format strings.
 
     :rtype: :py:class:`VertexDomain`
-    '''
-    attribute_usages = [create_attribute_usage(f) \
-                        for f in attribute_usage_formats]
+    """
+    attribute_usages = [create_attribute_usage(f) for f in attribute_usage_formats]
     return IndexedVertexDomain(attribute_usages)
 
-class VertexDomain(object):
-    '''Management of a set of vertex lists.
+
+class VertexDomain:
+    """Management of a set of vertex lists.
 
     Construction of a vertex domain is usually done with the
     :py:func:`create_domain` function.
-    '''
+    """
     _version = 0
     _initial_count = 16
 
@@ -173,8 +170,7 @@ class VertexDomain(object):
         attributes = []
         self.buffer_attributes = []   # list of (buffer, attributes)
         for attribute, usage, vbo in attribute_usages:
-            if (have_multi_texcoord and
-                isinstance(attribute, vertexattribute.TexCoordAttribute)):
+            if have_multi_texcoord and isinstance(attribute, vertexattribute.TexCoordAttribute):
                 attribute.convert_to_multi_tex_coord_attribute()
 
             if usage == GL_STATIC_DRAW:
@@ -189,8 +185,7 @@ class VertexDomain(object):
                     usage=usage, vbo=vbo)
                 attribute.buffer.element_size = attribute.stride
                 attribute.buffer.attributes = (attribute,)
-                self.buffer_attributes.append(
-                    (attribute.buffer, (attribute,)))
+                self.buffer_attributes.append((attribute.buffer, (attribute,)))
 
         # Create buffer for interleaved data
         if static_attributes:
@@ -199,8 +194,7 @@ class VertexDomain(object):
             buffer = vertexbuffer.create_mappable_buffer(
                 stride * self.allocator.capacity, usage=GL_STATIC_DRAW)
             buffer.element_size = stride
-            self.buffer_attributes.append(
-                (buffer, static_attributes))
+            self.buffer_attributes.append((buffer, static_attributes))
 
             attributes.extend(static_attributes)
             for attribute in static_attributes:
@@ -225,13 +219,11 @@ class VertexDomain(object):
                 if 'multi_tex_coords' not in self.attribute_names:
                     self.attribute_names['multi_tex_coords'] = []
                 assert texture not in self.attribute_names['multi_tex_coords'],\
-                    'More than one multi_tex_coord attribute for texture %d' % \
-                        texture
-                self.attribute_names['multi_tex_coords'].insert(texture,attribute)
+                    'More than one multi_tex_coord attribute for texture %d' % texture
+                self.attribute_names['multi_tex_coords'].insert(texture, attribute)
             else:
                 name = attribute.plural
-                assert name not in self.attributes, \
-                    'More than one "%s" attribute given' % name
+                assert name not in self.attributes, 'More than one "%s" attribute given' % name
                 self.attribute_names[name] = attribute
 
     def __del__(self):
@@ -244,7 +236,7 @@ class VertexDomain(object):
                 pass
 
     def _safe_alloc(self, count):
-        '''Allocate vertices, resizing the buffers if necessary.'''
+        """Allocate vertices, resizing the buffers if necessary."""
         try:
             return self.allocator.alloc(count)
         except allocation.AllocatorMemoryException as e:
@@ -256,7 +248,7 @@ class VertexDomain(object):
             return self.allocator.alloc(count)
 
     def _safe_realloc(self, start, count, new_count):
-        '''Reallocate vertices, resizing the buffers if necessary.'''
+        """Reallocate vertices, resizing the buffers if necessary."""
         try:
             return self.allocator.realloc(start, count, new_count)
         except allocation.AllocatorMemoryException as e:
@@ -268,19 +260,19 @@ class VertexDomain(object):
             return self.allocator.realloc(start, count, new_count)
 
     def create(self, count):
-        '''Create a :py:class:`VertexList` in this domain.
+        """Create a :py:class:`VertexList` in this domain.
 
         :Parameters:
             `count` : int
                 Number of vertices to create.
 
         :rtype: :py:class:`VertexList`
-        '''
+        """
         start = self._safe_alloc(count)
         return VertexList(self, start, count)
 
     def draw(self, mode, vertex_list=None):
-        '''Draw vertices in the domain.
+        """Draw vertices in the domain.
 
         If `vertex_list` is not specified, all vertices in the domain are
         drawn.  This is the most efficient way to render primitives.
@@ -294,7 +286,7 @@ class VertexDomain(object):
             `vertex_list` : `~pyglet.graphics.vertexdomain.VertexList`
                 Vertex list to draw, or ``None`` for all lists in this domain.
 
-        '''
+        """
         glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
         for buffer, attributes in self.buffer_attributes:
             buffer.bind()
@@ -330,13 +322,27 @@ class VertexDomain(object):
         return not self.allocator.starts
 
     def __repr__(self):
-        return '<%s@%x %s>' % (self.__class__.__name__, id(self),
-                               self.allocator)
+        return '<%s@%x %s>' % (self.__class__.__name__, id(self), self.allocator)
 
-class VertexList(object):
-    '''A list of vertices within a :py:class:`VertexDomain`.  Use
+
+class VertexList:
+    """A list of vertices within a :py:class:`VertexDomain`.  Use
     :py:meth:`VertexDomain.create` to construct this list.
-    '''
+    """
+    _colors_cache = None
+    _colors_cache_version = None
+    _fog_coords_cache = None
+    _fog_coords_cache_version = None
+    _edge_flags_cache = None
+    _edge_flags_cache_version = None
+    _normals_cache = None
+    _normals_cache_version = None
+    _secondary_colors_cache = None
+    _secondary_colors_cache_version = None
+    _tex_coords_cache = None
+    _tex_coords_cache_version = None
+    _vertices_cache = None
+    _vertices_cache_version = None
 
     def __init__(self, domain, start, count):
         # TODO make private
@@ -345,37 +351,37 @@ class VertexList(object):
         self.count = count
 
     def get_size(self):
-        '''Get the number of vertices in the list.
+        """Get the number of vertices in the list.
 
         :rtype: int
-        '''
+        """
         return self.count
 
     def get_domain(self):
-        '''Get the domain this vertex list belongs to.
+        """Get the domain this vertex list belongs to.
 
         :rtype: :py:class:`VertexDomain`
-        '''
+        """
         return self.domain
 
     def draw(self, mode):
-        '''Draw this vertex list in the given OpenGL mode.
+        """Draw this vertex list in the given OpenGL mode.
 
         :Parameters:
             `mode` : int
                 OpenGL drawing mode, e.g. ``GL_POINTS``, ``GL_LINES``, etc.
 
-        '''
+        """
         self.domain.draw(mode, self)
 
     def resize(self, count):
-        '''Resize this group.
+        """Resize this group.
 
         :Parameters:
             `count` : int
                 New number of vertices in the list.
 
-        '''
+        """
         new_start = self.domain._safe_realloc(self.start, self.count, count)
         if new_start != self.start:
             # Copy contents to new location
@@ -398,11 +404,11 @@ class VertexList(object):
         self._vertices_cache_version = None
 
     def delete(self):
-        '''Delete this group.'''
+        """Delete this group."""
         self.domain.allocator.dealloc(self.start, self.count)
 
     def migrate(self, domain):
-        '''Move this group from its current domain and add to the specified
+        """Move this group from its current domain and add to the specified
         one.  Attributes on domains must match.  (In practice, used to change
         parent state of some vertices).
 
@@ -410,17 +416,15 @@ class VertexList(object):
             `domain` : `VertexDomain`
                 Domain to migrate this vertex list to.
 
-        '''
+        """
         assert list(domain.attribute_names.keys()) == \
             list(self.domain.attribute_names.keys()), 'Domain attributes must match.'
 
         new_start = domain._safe_alloc(self.count)
         for key, old_attribute in self.domain.attribute_names.items():
-            old = old_attribute.get_region(old_attribute.buffer,
-                                           self.start, self.count)
+            old = old_attribute.get_region(old_attribute.buffer, self.start, self.count)
             new_attribute = domain.attribute_names[key]
-            new = new_attribute.get_region(new_attribute.buffer,
-                                           new_start, self.count)
+            new = new_attribute.get_region(new_attribute.buffer, new_start, self.count)
             new.array[:] = old.array[:]
             new.invalidate()
 
@@ -443,32 +447,27 @@ class VertexList(object):
         region.array[:] = data
         region.invalidate()
 
-    # ---
-
-    def _get_colors(self):
-        if (self._colors_cache_version != self.domain._version):
+    @property
+    def colors(self):
+        """Array of color data."""
+        if self._colors_cache_version != self.domain._version:
             domain = self.domain
             attribute = domain.attribute_names['colors']
-            self._colors_cache = attribute.get_region(
-                attribute.buffer, self.start, self.count)
+            self._colors_cache = attribute.get_region(attribute.buffer, self.start, self.count)
             self._colors_cache_version = domain._version
 
         region = self._colors_cache
         region.invalidate()
         return region.array
 
-    def _set_colors(self, data):
-        self._get_colors()[:] = data
+    @colors.setter
+    def colors(self, data):
+        self.colors[:] = data
 
-    _colors_cache = None
-    _colors_cache_version = None
-    colors = property(_get_colors, _set_colors,
-                      doc='''Array of color data.''')
-
-    # ---
-
-    def _get_fog_coords(self):
-        if (self._fog_coords_cache_version != self.domain._version):
+    @property
+    def fog_coords(self):
+        """Array of fog coordinate data."""
+        if self._fog_coords_cache_version != self.domain._version:
             domain = self.domain
             attribute = domain.attribute_names['fog_coords']
             self._fog_coords_cache = attribute.get_region(
@@ -479,18 +478,14 @@ class VertexList(object):
         region.invalidate()
         return region.array
 
-    def _set_fog_coords(self, data):
-        self._get_fog_coords()[:] = data
+    @fog_coords.setter
+    def fog_coords(self, data):
+        self.fog_coords[:] = data
 
-    _fog_coords_cache = None
-    _fog_coords_cache_version = None
-    fog_coords = property(_get_fog_coords, _set_fog_coords,
-                          doc='''Array of fog coordinate data.''')
-
-    # ---
-
-    def _get_edge_flags(self):
-        if (self._edge_flags_cache_version != self.domain._version):
+    @property
+    def edge_flags(self):
+        """Array of edge flag data."""
+        if self._edge_flags_cache_version != self.domain._version:
             domain = self.domain
             attribute = domain.attribute_names['edge_flags']
             self._edge_flags_cache = attribute.get_region(
@@ -501,18 +496,14 @@ class VertexList(object):
         region.invalidate()
         return region.array
 
-    def _set_edge_flags(self, data):
-        self._get_edge_flags()[:] = data
+    @edge_flags.setter
+    def edge_flags(self, data):
+        self.edge_flags[:] = data
 
-    _edge_flags_cache = None
-    _edge_flags_cache_version = None
-    edge_flags = property(_get_edge_flags, _set_edge_flags,
-                          doc='''Array of edge flag data.''')
-
-    # ---
-
-    def _get_normals(self):
-        if (self._normals_cache_version != self.domain._version):
+    @property
+    def normals(self):
+        """Array of normal vector data."""
+        if self._normals_cache_version != self.domain._version:
             domain = self.domain
             attribute = domain.attribute_names['normals']
             self._normals_cache = attribute.get_region(
@@ -523,18 +514,14 @@ class VertexList(object):
         region.invalidate()
         return region.array
 
-    def _set_normals(self, data):
-        self._get_normals()[:] = data
+    @normals.setter
+    def normals(self, data):
+        self.normals[:] = data
 
-    _normals_cache = None
-    _normals_cache_version = None
-    normals = property(_get_normals, _set_normals,
-                       doc='''Array of normal vector data.''')
-
-    # ---
-
-    def _get_secondary_colors(self):
-        if (self._secondary_colors_cache_version != self.domain._version):
+    @property
+    def secondary_colors(self):
+        """Array of secondary color data."""
+        if self._secondary_colors_cache_version != self.domain._version:
             domain = self.domain
             attribute = domain.attribute_names['secondary_colors']
             self._secondary_colors_cache = attribute.get_region(
@@ -545,22 +532,15 @@ class VertexList(object):
         region.invalidate()
         return region.array
 
-    def _set_secondary_colors(self, data):
-        self._get_secondary_colors()[:] = data
+    @secondary_colors.setter
+    def secondary_colors(self, data):
+        self.secondary_colors[:] = data
 
-    _secondary_colors_cache = None
-    _secondary_colors_cache_version = None
-    secondary_colors = property(_get_secondary_colors, _set_secondary_colors,
-                                doc='''Array of secondary color data.''')
-
-    # ---
-
-    _tex_coords_cache = None
-    _tex_coords_cache_version = None
-
-    def _get_tex_coords(self):
+    @property
+    def tex_coords(self):
+        """Array of texture coordinate data."""
         if 'multi_tex_coords' not in self.domain.attribute_names:
-            if (self._tex_coords_cache_version != self.domain._version):
+            if self._tex_coords_cache_version != self.domain._version:
                 domain = self.domain
                 attribute = domain.attribute_names['tex_coords']
                 self._tex_coords_cache = attribute.get_region(
@@ -573,18 +553,16 @@ class VertexList(object):
         else:
             return None
 
-    def _set_tex_coords(self, data):
-        if self._get_tex_coords() != None:
-            self._get_tex_coords()[:] = data
+    @tex_coords.setter
+    def tex_coords(self, data):
+        if self.tex_coords:
+            self.tex_coords[:] = data
 
-    tex_coords = property(_get_tex_coords, _set_tex_coords,
-                          doc='''Array of texture coordinate data.''')
-
-    # ---
-
-    def _get_multi_tex_coords(self):
+    @property
+    def multi_tex_coords(self):
+        """Multi-array texture coordinate data."""
         if 'tex_coords' not in self.domain.attribute_names:
-            if (self._tex_coords_cache_version != self.domain._version):
+            if self._tex_coords_cache_version != self.domain._version:
                 domain = self.domain
                 attribute = domain.attribute_names['multi_tex_coords']
                 self._tex_coords_cache = []
@@ -602,24 +580,19 @@ class VertexList(object):
         else:
             return None
 
-    def _set_multi_tex_coords(self, data):
-        if self._get_multi_tex_coords() != None:
-            for a in range(0, len(self._tex_coords_cache),1):
+    @multi_tex_coords.setter
+    def multi_tex_coords(self, data):
+        if self.multi_tex_coords:
+            for a in range(0, len(self._tex_coords_cache), 1):
                 if a > len(data):
                     break
-                elif data[a] != None:
+                elif data[a]:
                     self._tex_coords_cache[a].array[:] = data[a]
 
-    multi_tex_coords = property(_get_multi_tex_coords, _set_multi_tex_coords,
-                                doc='''Multi-array texture coordinate data.''')
-
-    # ---
-
-    _vertices_cache = None
-    _vertices_cache_version = None
-
-    def _get_vertices(self):
-        if (self._vertices_cache_version != self.domain._version):
+    @property
+    def vertices(self):
+        """Array of vertex coordinate data."""
+        if self._vertices_cache_version != self.domain._version:
             domain = self.domain
             attribute = domain.attribute_names['vertices']
             self._vertices_cache = attribute.get_region(
@@ -630,18 +603,17 @@ class VertexList(object):
         region.invalidate()
         return region.array
 
-    def _set_vertices(self, data):
-        self._get_vertices()[:] = data
+    @vertices.setter
+    def vertices(self, data):
+        self.vertices[:] = data
 
-    vertices = property(_get_vertices, _set_vertices,
-                        doc='''Array of vertex coordinate data.''')
 
 class IndexedVertexDomain(VertexDomain):
-    '''Management of a set of indexed vertex lists.
+    """Management of a set of indexed vertex lists.
 
     Construction of an indexed vertex domain is usually done with the
     `create_indexed_domain` function.
-    '''
+    """
     _initial_index_count = 16
 
     def __init__(self, attribute_usages, index_gl_type=GL_UNSIGNED_INT):
@@ -657,7 +629,7 @@ class IndexedVertexDomain(VertexDomain):
             target=GL_ELEMENT_ARRAY_BUFFER)
 
     def _safe_index_alloc(self, count):
-        '''Allocate indices, resizing the buffers if necessary.'''
+        """Allocate indices, resizing the buffers if necessary."""
         try:
             return self.index_allocator.alloc(count)
         except allocation.AllocatorMemoryException as e:
@@ -668,7 +640,7 @@ class IndexedVertexDomain(VertexDomain):
             return self.index_allocator.alloc(count)
 
     def _safe_index_realloc(self, start, count, new_count):
-        '''Reallocate indices, resizing the buffers if necessary.'''
+        """Reallocate indices, resizing the buffers if necessary."""
         try:
             return self.index_allocator.realloc(start, count, new_count)
         except allocation.AllocatorMemoryException as e:
@@ -679,7 +651,7 @@ class IndexedVertexDomain(VertexDomain):
             return self.index_allocator.realloc(start, count, new_count)
 
     def create(self, count, index_count):
-        '''Create an :py:class:`IndexedVertexList` in this domain.
+        """Create an :py:class:`IndexedVertexList` in this domain.
 
         :Parameters:
             `count` : int
@@ -687,13 +659,13 @@ class IndexedVertexDomain(VertexDomain):
             `index_count`
                 Number of indices to create
 
-        '''
+        """
         start = self._safe_alloc(count)
         index_start = self._safe_index_alloc(index_count)
         return IndexedVertexList(self, start, count, index_start, index_count)
 
     def get_index_region(self, start, count):
-        '''Get a region of the index buffer.
+        """Get a region of the index buffer.
 
         :Parameters:
             `start` : int
@@ -702,14 +674,14 @@ class IndexedVertexDomain(VertexDomain):
                 Number of indices to map.
 
         :rtype: Array of int
-        '''
+        """
         byte_start = self.index_element_size * start
         byte_count = self.index_element_size * count
         ptr_type = ctypes.POINTER(self.index_c_type * count)
         return self.index_buffer.get_region(byte_start, byte_count, ptr_type)
 
     def draw(self, mode, vertex_list=None):
-        '''Draw vertices in the domain.
+        """Draw vertices in the domain.
 
         If `vertex_list` is not specified, all vertices in the domain are
         drawn.  This is the most efficient way to render primitives.
@@ -723,7 +695,7 @@ class IndexedVertexDomain(VertexDomain):
             `vertex_list` : `IndexedVertexList`
                 Vertex list to draw, or ``None`` for all lists in this domain.
 
-        '''
+        """
         glPushClientAttrib(GL_CLIENT_VERTEX_ARRAY_BIT)
         for buffer, attributes in self.buffer_attributes:
             buffer.bind()
@@ -736,8 +708,8 @@ class IndexedVertexDomain(VertexDomain):
 
         if vertex_list is not None:
             glDrawElements(mode, vertex_list.index_count, self.index_gl_type,
-                self.index_buffer.ptr +
-                    vertex_list.index_start * self.index_element_size)
+                           self.index_buffer.ptr + vertex_list.index_start *
+                           self.index_element_size)
         else:
             starts, sizes = self.index_allocator.get_allocated_regions()
             primcount = len(starts)
@@ -746,7 +718,7 @@ class IndexedVertexDomain(VertexDomain):
             elif primcount == 1:
                 # Common case
                 glDrawElements(mode, sizes[0], self.index_gl_type,
-                    self.index_buffer.ptr + starts[0])
+                               self.index_buffer.ptr + starts[0] * self.index_element_size)
             elif gl_info.have_version(1, 4):
                 starts = [s * self.index_element_size + self.index_buffer.ptr for s in starts]
                 starts = (ctypes.POINTER(GLvoid) * primcount)(*(GLintptr * primcount)(*starts))
@@ -755,18 +727,21 @@ class IndexedVertexDomain(VertexDomain):
             else:
                 for start, size in zip(starts, sizes):
                     glDrawElements(mode, size, self.index_gl_type,
-                        self.index_buffer.ptr +
-                            start * self.index_element_size)
+                                   self.index_buffer.ptr + start * self.index_element_size)
 
         self.index_buffer.unbind()
         for buffer, _ in self.buffer_attributes:
             buffer.unbind()
         glPopClientAttrib()
 
+
 class IndexedVertexList(VertexList):
-    '''A list of vertices within an :py:class:`IndexedVertexDomain` that are
+    """A list of vertices within an :py:class:`IndexedVertexDomain` that are
     indexed. Use :py:meth:`IndexedVertexDomain.create` to construct this list.
-    '''
+    """
+    _indices_cache = None
+    _indices_cache_version = None
+
     def __init__(self, domain, start, count, index_start, index_count):
         super(IndexedVertexList, self).__init__(domain, start, count)
 
@@ -777,7 +752,7 @@ class IndexedVertexList(VertexList):
         self.domain.draw(mode, self)
 
     def resize(self, count, index_count):
-        '''Resize this group.
+        """Resize this group.
 
         :Parameters:
             `count` : int
@@ -785,7 +760,7 @@ class IndexedVertexList(VertexList):
             `index_count` : int
                 New number of indices in the list.
 
-        '''
+        """
         old_start = self.start
         super(IndexedVertexList, self).resize(count)
 
@@ -809,12 +784,12 @@ class IndexedVertexList(VertexList):
         self._indices_cache_version = None
 
     def delete(self):
-        '''Delete this group.'''
+        """Delete this group."""
         super(IndexedVertexList, self).delete()
         self.domain.index_allocator.dealloc(self.index_start, self.index_count)
-        
+
     def migrate(self, domain):
-        '''Move this group from its current indexed domain and add to the 
+        """Move this group from its current indexed domain and add to the 
         specified one.  Attributes on domains must match.  (In practice, used 
         to change parent state of some vertices).
 
@@ -822,7 +797,7 @@ class IndexedVertexList(VertexList):
             `domain` : `IndexedVertexDomain`
                 Indexed domain to migrate this vertex list to.
 
-        '''
+        """
         old_start = self.start
         old_domain = self.domain
         super(IndexedVertexList, self).migrate(domain)
@@ -831,12 +806,11 @@ class IndexedVertexList(VertexList):
         # because the vertices are in a new position in the new domain
         if old_start != self.start:
             diff = self.start - old_start
-            region = old_domain.get_index_region(self.index_start, 
-                                self.index_count)
+            region = old_domain.get_index_region(self.index_start, self.index_count)
             old_indices = region.array
             old_indices[:] = [i + diff for i in old_indices]
             region.invalidate()
-                                                
+
         # copy indices to new domain
         old = old_domain.get_index_region(self.index_start, self.index_count)
         # must delloc before calling safe_index_alloc or else problems when same
@@ -857,9 +831,9 @@ class IndexedVertexList(VertexList):
         region.array[:] = data
         region.invalidate()
 
-    # ---
-
-    def _get_indices(self):
+    @property
+    def indices(self):
+        """Array of index data."""
         if self._indices_cache_version != self.domain._version:
             domain = self.domain
             self._indices_cache = domain.get_index_region(
@@ -870,10 +844,6 @@ class IndexedVertexList(VertexList):
         region.invalidate()
         return region.array
 
-    def _set_indices(self, data):
-        self._get_indices()[:] = data
-
-    _indices_cache = None
-    _indices_cache_version = None
-    indices = property(_get_indices, _set_indices,
-                       doc='''Array of index data.''')
+    @indices.setter
+    def indices(self, data):
+        self.indices[:] = data
