@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2019 pyglet contributors
+# Copyright (c) 2008-2020 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -80,10 +80,6 @@ handlers.
 .. versionadded:: 1.2
 
 """
-from __future__ import absolute_import
-
-__docformat__ = 'restructuredtext'
-__version__ = '$Id: $'
 
 import sys
 
